from dynaconf import Dynaconf
from importlib import resources


with resources.path("pygradus", "settings.toml") as path:
    settings = Dynaconf(
        envvar_prefix="DYNACONF",
        settings_files=[path],
    )

# `envvar_prefix` = export envvars with `export DYNACONF_FOO=bar`.
# `settings_files` = Load these files in the order.



import requests
from copy import deepcopy
from pygradus.config import settings as st


HEADERS = {
    "accept": "application/json",
}

API_MODE = {"post": requests.post, "get": requests.get}


def send_request(
    mode: str,
    url: str,
    data: dict = None,
    json: dict = None,
    token: str = "",
):

    header = deepcopy(HEADERS)
    header["Content-Type"] = st[mode]["content-type"]
    if token:
        header["Authorization"] = token

    if url is None:
        url = st.BASE_URL
    url += st[mode]["url"]

    func = API_MODE[st[mode]["mode"]]

    response = func(url, json=json, data=data, headers=header)
    return response



import re
import json
from pathlib import Path
import nbformat.v4 as nbf
from nbformat import write as nbf_write

from pygradus.config import settings as st

start_re = re.compile(st.START_RE)
end_re = re.compile(st.END_RE)
delete_re = re.compile(st.DEL_RE)
tasks_re = re.compile(st.TASKS_RE)


def public_version(input: str, output: str):

    filename = Path(input)
    with open(filename, "r") as fh:
        data = json.load(fh)

    cells = data["cells"]

    nb = nbf.new_notebook()

    tasks_content = None
    new_cells = []
    for c in cells:
        src = c["source"]
        start = None
        end = None
        delete = False
        tasks = False
        for i, l in enumerate(src):
            # print(i, l)
            if start_re.search(l):
                start = i
            if end_re.search(l):
                end = i
            if delete_re.search(l):
                delete = True
            if tasks_re.search(l):
                tasks = True

        if start is not None and end is not None:
            new_src = src[:start] + ["    # Write your code here\n"] + src[end + 1 :]
        else:
            new_src = src

        if tasks:
            tasks_content = src

        if delete or tasks:
            pass
        elif c["cell_type"] == "markdown":
            cell = nbf.new_markdown_cell(new_src)
            new_cells.append(cell)
        elif c["cell_type"] == "code":
            cell = nbf.new_code_cell(new_src)
            new_cells.append(cell)

    tasks_content = "".join(tasks_content[1:]).replace("TASKS = ", "")

    submission = f"""
proposed_solution = {{
        'attempt': {{
        'course_name': COURSE_NAME,
        'exercise_name': EXERCISE_NAME,
        'username': STUDENT_NAME,
    }},
    'task_attempts': {tasks_content}

    }}
import pygradus as gr
gr.check_solution(proposed_solution)
    """
    cell = nbf.new_code_cell(submission)
    new_cells.append(cell)

    nb["cells"] = new_cells

    with open(output, "w") as f:
        nbf_write(nb, f, 4)


if __name__ == "__main__":
    import sys

    input = sys.argv[1]
    output = sys.argv[2]
    public_version(input, output)



from pygradus.api import send_request
from pygradus.config import settings as st


def check_solution(proposal, url=st.BASE_URL):

    r = send_request("attempt", url, json=proposal)
    if r.status_code != 200:
        return r.text
    res = r.json()

    max_len = 50
    answers = res["task_attempts"]
    title = f"Total Correct Answers {res['total_correct']} / {len(answers)}"
    row = f"|{{0:^{max_len}}}|{{1:^20}}|"
    sep = row.format("-" * max_len, "-" * 20)
    print(row.format("Task Name", "Status"))
    print(sep)
    print(sep)
    for a in answers:
        print(
            row.format(
                a["name"][:max_len], "Correct" if a["is_correct"] else "Incorrect"
            )
        )
        print(sep)



from getpass import getpass
from pygradus.config import settings as st
from pygradus.api import send_request


def create_exercise(username, config, url=st.BASE_URL):

    password = getpass()
    login_data = {"username": username, "password": password}
    r = send_request("token", url, data=login_data)
    if r.status_code == 200:
        token = f"Bearer {r.json()['access_token']}"
    else:
        raise Exception(r.text)

    # Get existing user data
    r = send_request("users", url, token=token)
    if r.status_code == 200:
        user_data = r.json()
    else:
        raise Execption("Failed to get user data")

    course = config["course_name"]
    my_course = {}
    for c in user_data["courses"]:
        if course == c["name"]:  # the course is already mine
            my_course = c
            break
    else:
        course = {"name": course}
        r = send_request("course", url, json=course, token=token)
        if r.status_code != 200:
            return r.text

        my_course = r.json()

    exercise = config["exercise_name"]
    data = {"course_name": course, "name": exercise}
    my_exercise = {}
    for ex in my_course["exercises"]:
        if exercise == ex["name"]:
            my_exercise = ex
            break
    else:
        r = send_request("exercise", url, json=data, token=token)
        if r.status_code != 200:
            return r.text
        my_exercise = r.json()

    tasks = config["tasks"]
    my_tasks = my_exercise["tasks"]
    data = {"course_name": course, "exercise_name": exercise}
    for t in tasks:
        payload = {
            **data,
            **t,
            "disabled": False,
        }
        r = send_request("task", url, json=payload, token=token)
        if r.status_code != 200:
            return r.text

    # disable missing tasks
    new_tasks_names = [t["name"] for t in tasks]
    for t in my_tasks:
        if t["name"] not in new_tasks_names:
            payload = {
                **t,
                "disabled": True,
            }
            r = send_request("task", url, json=payload, token=token)
            if r.status_code != 200:
                return r.text

    r = send_request("users", url, token=token)
    if r.status_code == 200:
        user_data = r.json()
        print("Exercise correctly updated")
    else:
        return r.text

    return user_data
